<?php

namespace App\Facade\Filters;
use App\Facade\Filters\Pagination;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Validation\Rule;
use JsonMapper;

class Filters
{

    public const FILTERS_VALIDATION = [];
    //     'filters.*.column' => [ 'required', 'string' ],
    //     'filters.*.type' => [ 'required', Rule::enum(FilterTypeEnum::class) ],
    //     'filters.*.filter' => [ 'required', 'string' ],

    //     'orders.*.by' => [ 'required', 'string' ],
    //     'orders.*.sort' => [ 'required', Rule::enum(OrderTypeEnum::class) ],

    //     'pagination.size' => 'number',
    //     'pagination.page' => 'number',
    // ];

    /**
     * @var Filter[]
     */
    public array $filters;

    /**
     * @var Order[]
     */
    public array $orders;

    public ?Pagination $pagination;

    public static function fromArrayOrObject(array | object $data): Filters
    {
        $mapper = new JsonMapper();
        if (is_array($data)) {
            $data = json_decode(json_encode($data), false);
        }
        
        $data->filters ??= [];
        $data->orders ??= [];
        $data->pagination ??= null;

        return $mapper->map($data, \App\Facade\Filters\Filters::class);
    }

    public function apply(Builder $builder)
    {
        foreach ($this->filters as $filter) {
            switch ($filter->type) {
                case FilterTypeEnum::Is:
                    $builder->where($filter->column, $filter->filter);
                    break;
                case FilterTypeEnum::Like:
                    $builder->whereLike($filter->column, $filter->filter);
                    break;
            }
        }

        foreach ($this->orders as $order) {
            $builder->orderBy($order->by, $order->sort->value);
        }

        if (!is_null($this->pagination)) {
            $builder->paginate($this->pagination->size , ['*'], 'page', $this->pagination->page);
        }
    }
}